# PowerShell script for building and deploying arm_am-pom to JBoss EAP 7.4

# Load configuration from JSON file
$config = Get-Content -Raw -Path "$PSScriptRoot\config.json" | ConvertFrom-Json

# Configuration variables from JSON
$JBOSS_HOME = $config.jbossHome
$PROJECT_DIR = $config.ear.projectDir
$DEPLOYMENT_DIR = $config.deploymentDir
$MAVEN_PROFILE = $config.ear.mavenProfile
$JAVA_HOME = $config.javaHome

# Store the original directory
$originalDirectory = Get-Location

# Function to check if JBoss is running
function Test-JBossRunning {
    $jbossProcess = Get-CimInstance Win32_Process -Filter "Name = 'java.exe'" | 
                    Where-Object { $_.CommandLine -like "*jboss.home.dir=$JBOSS_HOME*" }
    return $null -ne $jbossProcess
}

Write-Host "Starting build and deploy process..." -ForegroundColor Green

# Check if Java exists
if (-not (Test-Path $JAVA_HOME)) {
    Write-Host "Error: Java 8 not found at $JAVA_HOME" -ForegroundColor Red
    Set-Location $originalDirectory  # Restore original directory
    exit 1
}

# Set JAVA_HOME for the build
$env:JAVA_HOME = $JAVA_HOME
Write-Host "Using Java from: $JAVA_HOME" -ForegroundColor Yellow

# Check if JBoss directory exists
if (-not (Test-Path $JBOSS_HOME)) {
    Write-Host "Error: JBoss directory not found at $JBOSS_HOME" -ForegroundColor Red
    Set-Location $originalDirectory  # Restore original directory
    exit 1
}

# Check if project directory exists
if (-not (Test-Path $PROJECT_DIR)) {
    Write-Host "Error: Project directory not found at $PROJECT_DIR" -ForegroundColor Red
    Set-Location $originalDirectory  # Restore original directory
    exit 1
}

# Navigate to project directory
Set-Location $PROJECT_DIR

# Clean and build the project with Maven
Write-Host "Building project with Maven using profile $MAVEN_PROFILE..." -ForegroundColor Yellow
Write-Host "Current directory: $(Get-Location)" -ForegroundColor Yellow

# First build the parent and modules with debug output
Write-Host "Building parent POM..." -ForegroundColor Yellow
& "$env:JAVA_HOME\bin\java" -version
mvn clean install -N -P$MAVEN_PROFILE
if ($LASTEXITCODE -ne 0) {
    Write-Host "Maven parent build failed!" -ForegroundColor Red
    Set-Location $originalDirectory  # Restore original directory
    exit 1
}

# Build the EJB module first with debug output
Write-Host "Building EJB module..." -ForegroundColor Yellow
Set-Location "$PROJECT_DIR\arm_am-ejb"
mvn clean install -DskipTests -P$MAVEN_PROFILE
if ($LASTEXITCODE -ne 0) {
    Write-Host "Maven EJB module build failed!" -ForegroundColor Red
    Set-Location $originalDirectory  # Restore original directory
    exit 1
}

# Build the WAR module with debug output
Write-Host "Building WAR module..." -ForegroundColor Yellow
Set-Location "$PROJECT_DIR\arm_am"
mvn clean install -DskipTests -P$MAVEN_PROFILE
if ($LASTEXITCODE -ne 0) {
    Write-Host "Maven WAR module build failed!" -ForegroundColor Red
    Set-Location $originalDirectory  # Restore original directory
    exit 1
}

# Finally build the EAR module with debug output
Write-Host "Building EAR module..." -ForegroundColor Yellow
Set-Location "$PROJECT_DIR\arm_am-ear"
mvn clean package -DskipTests -P$MAVEN_PROFILE
if ($LASTEXITCODE -ne 0) {
    Write-Host "Maven EAR module build failed!" -ForegroundColor Red
    Set-Location $originalDirectory  # Restore original directory
    exit 1
}

# Check if build produced the EAR file
$earFile = Get-ChildItem -Path "$PROJECT_DIR\arm_am-ear\target\*.ear" -ErrorAction SilentlyContinue | Select-Object -First 1

if (-not $earFile) {
    Write-Host "Error: EAR file not found after build" -ForegroundColor Red
    Write-Host "Searching in: $PROJECT_DIR\arm_am-ear\target\" -ForegroundColor Yellow
    Set-Location $originalDirectory  # Restore original directory
    exit 1
}

# Create deployments directory if it doesn't exist
if (-not (Test-Path $DEPLOYMENT_DIR)) {
    New-Item -ItemType Directory -Path $DEPLOYMENT_DIR
}

# Copy EAR file to JBoss deployments directory
Write-Host "Deploying EAR to JBoss..." -ForegroundColor Yellow
Write-Host "Copying from: $($earFile.FullName)" -ForegroundColor Yellow
Write-Host "Copying to: $DEPLOYMENT_DIR" -ForegroundColor Yellow
Copy-Item $earFile.FullName $DEPLOYMENT_DIR

# Check if JBoss is running
if (Test-JBossRunning) {
    Write-Host "JBoss is running - deployment will be automatically picked up" -ForegroundColor Green
} else {
    Write-Host "Note: JBoss is not running. Start JBoss to complete deployment" -ForegroundColor Yellow
}

Write-Host "Build and deploy process completed!" -ForegroundColor Green

# Restore the original directory
Set-Location $originalDirectory
